import os
import subprocess
import datetime

#CONSTANTS
TIMESTAMP = datetime.datetime.utcnow()
NUTFACTORY_PATH = os.path.dirname(os.path.realpath(__file__))+"/"
MAIN_LOGFILENAME = "MAIN_OPERATIONS_LOG_"
DOXYGEN_REPORT_FILENAME = "DOXYGEN_OUTPUT_FOR_"
GIT_REPORT_FILENAME = "GIT_OUTPUT_FOR_"
SCP_REPORT_FILENAME = "SCP_OUTPUT_FOR_"
DOXYGEN_BASELINE_FILE = "NUTS-doxygen-baseline"
GITREPOLIST = "gitlist.txt"
CONFIG_FOLDER = "/config/"
SCP_TARGET = "torleift@login.idi.ntnu.no:./public_html/NUTS/"
#ENUMS
LOG_TYPE_MAIN_OP = 1
LOG_TYPE_DOXY_OP = 2
LOG_TYPE_GIT_OP = 3
LOG_TYPE_SCP_OP	= 4

def current_timestamp():
    return datetime.datetime.utcnow()

def current_timestamp_string():
    return current_timestamp().strftime('%Y-%m-%d_%H-%M-%S')

def operation_log(log, logtype):
    #Check if the main log directory exsists
    logdir = NUTFACTORY_PATH+"/log/"
    if logtype == LOG_TYPE_MAIN_OP:
        logfilename_withpath = logdir+MAIN_LOGFILENAME+TIMESTAMP.strftime('%Y-%m-%d_%H-%M-%S')
    elif logtype == LOG_TYPE_DOXY_OP:
        logfilename_withpath = logdir+DOXYGEN_REPORT_FILENAME+TIMESTAMP.strftime('%Y-%m-%d_%H-%M-%S')
    elif logtype == LOG_TYPE_GIT_OP:
        logfilename_withpath = logdir+GIT_REPORT_FILENAME+TIMESTAMP.strftime('%Y-%m-%d_%H-%M-%S') 
    elif logtype == LOG_TYPE_SCP_OP:
        logfilename_withpath = logdir+SCP_REPORT_FILENAME+TIMESTAMP.strftime('%Y-%m-%d_%H-%M-%S')
    else:
        logfilename_withpath = logdir+MAIN_LOGFILENAME+TIMESTAMP.strftime('%Y-%m-%d_%H-%M-%S')
    if os.path.isdir(logdir):
        if os.path.isfile(logfilename_withpath):
            f = open(logfilename_withpath, 'a')
            f.write("\n"+log+"\n")
            f.close()
        else:
            f = open(logfilename_withpath, "w+")
            f.write("\n"+log+"\n")
            f.close()
    else:
        createLogDir = subprocess.Popen('mkdir '+ logdir, shell=True)
        createLogDir.wait()
        f = open(logfilename_withpath, "w+")
        f.write("\n"+log+"\n")
        f.close()


def check_gitlist():
    f = open(NUTFACTORY_PATH+"/"+CONFIG_FOLDER+"/"+GITREPOLIST)
    lines = f.readlines()
    f.close()
    repos = {}
    for line in lines:
        if line[0] != '#':
            repo = line.split()
            if(len(repo) == 1):
                repos[repo[0]] = ""
            elif(len(repo) == 2):
                repos[repo[0]] = repo[1]
    return repos


def scp_result(foldername):
    scp_cmd = "scp -rp " + foldername + " " + SCP_TARGET 
    process = subprocess.Popen(scp_cmd, shell=True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr=subprocess.PIPE,)
    process.wait()
    output = process.communicate()[0]
    operation_log(output, LOG_TYPE_SCP_OP)
    

def doxygen_process(foldername, sourcefolder):
    doxyreportfoldername = foldername+"DoxyReport"
    #Baseline file for Doxygen config
    doxygen_baseline_path = NUTFACTORY_PATH+CONFIG_FOLDER+DOXYGEN_BASELINE_FILE
    baseline = open(doxygen_baseline_path, "r+")
    lines = baseline.readlines()
    baseline.close()
    #Create a repo specific doxygen config file
    doxyfileName = foldername+"DoxyGenFile"
    doxyfilePath = NUTFACTORY_PATH+"/"+foldername+"/"+sourcefolder+"/"+doxyfileName
    doxyfile = open(doxyfilePath, "w+")
    for line in lines:
        if "PROJECT_NAME" in line:
            line = "PROJECT_NAME    = " + foldername+ "\n"
        if "OUTPUT_DIRECTORY" in line:
            line = "OUTPUT_DIRECTORY    = " + NUTFACTORY_PATH +"/"+doxyreportfoldername+"/   \n"
        if "INPUT" in line:
            line = "INPUT   = " + NUTFACTORY_PATH + "/"+foldername+"/"+sourcefolder+"/   \n"
	if "PROJECT_LOGO" in line:
	    line = "PROJECT LOGO = " +  NUTFACTORY_PATH + CONFIG_FOLDER + "logo.png \n"
        doxyfile.write(line)
    doxyfile.close()
    remove_directory(doxyreportfoldername)

    make_directory(NUTFACTORY_PATH+"/"+doxyreportfoldername)
    process = subprocess.Popen('doxygen ' + doxyfilePath, 
                                shell=True,
                                stdin=subprocess.PIPE,
                                stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,)

    output = process.communicate()[0]
    operation_log(output, LOG_TYPE_DOXY_OP)
    scp_result(NUTFACTORY_PATH+"/"+doxyreportfoldername)


def remove_directory(folderPath):
    process = subprocess.Popen('rm -rf ' + folderPath,
                         shell=True, 
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE,)
    string = "\n Deleting folder: "+folderPath +" at " + current_timestamp_string() +"\n"
    output = process.communicate()[0]
    
    if len(output) > 0:
        operation_log(string + "\t message from system: " + output, LOG_TYPE_MAIN_OP)
    else:
        operation_log(string, LOG_TYPE_MAIN_OP)


def make_directory(folderPath):
    process = subprocess.Popen('mkdir ' + folderPath,
                         shell=True, 
                         stdin=subprocess.PIPE,
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE,)
    string = "\n Creating folder: "+folderPath +" at " + current_timestamp_string() +"\n"
    output = process.communicate()[0]
    
    if len(output) > 0:
        operation_log(string + "\t message from system: " + output, LOG_TYPE_MAIN_OP)
    else:
        operation_log(string, LOG_TYPE_MAIN_OP)


def git_download():
    repos = check_gitlist()
    operation_log("Starting to download repos at "+ current_timestamp_string(), LOG_TYPE_MAIN_OP)
    for key, value in repos.items():
        temp = key.split('/')
        foldername = temp[len(temp)-1]
        foldername = foldername[:-4]
        make_directory(NUTFACTORY_PATH+foldername)
        git_cmd = "git clone "+key + " " + NUTFACTORY_PATH + "/" + foldername
        process = subprocess.Popen(git_cmd, shell=True, 
                                    stdin=subprocess.PIPE,
                                   stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE,)
        process.wait()
        proc_output = process.communicate()[0]
        operation_log(proc_output, LOG_TYPE_GIT_OP)
        operation_log("Attempting to generate Doxygen report for "+foldername+" at "+ current_timestamp_string(), LOG_TYPE_MAIN_OP)
        doxygen_process(foldername, value)
        remove_directory(NUTFACTORY_PATH+foldername)

git_download()
exit()
