#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#define N  25000000
		   
//Some precalculated values
double pi;
double invsqrt2pi;
//Global variables!
int thread_count;
pthread_mutex_t mutex1;


struct arguments
{
	double start;
	double end;
	double *final_res;
	int num_pieces;
};

//Proto-types//
void createArguments(struct arguments *args, double *final_res, double start, double end, int pieces);
void* integral(void *arg);
double f(double x);
int divideWork(int thread_count); 
double simpson(double x1,double x2,double x3);



int main(int argc, char* argv[])
{
	double final_result;
	long thread;
	if(argc < 2)
	{
		printf("No specified number of threads, defaulting to 2 threads\n");
		thread_count = 2;
	}
	else
		thread_count = strtol(argv[1], NULL, 10);
	
	pthread_mutex_init(&mutex1, NULL);
	pthread_t* thread_handles;
	thread_handles = malloc(thread_count*sizeof(thread_handles));

	pi=2*acos(0);
	invsqrt2pi=1/sqrt(2*pi);
	
	int thread_work_part = divideWork(thread_count);
	double start = -1;
	double sub_parts = (double)2/thread_count; // Total length from -1 to 1 is 2.

	for(thread = 0; thread < thread_count; thread ++)
	{
		double start_pos = (start + (sub_parts*(int)thread)); 
		struct arguments *arg;
		arg = malloc(sizeof(*arg));
		//Note: We pass along a pointer to the final_result here
		createArguments(arg ,&final_result,  start_pos, start_pos + sub_parts, thread_work_part);
		pthread_create(&thread_handles[thread], NULL, integral, (void*)arg);
	}
		
	//Join 'em
	for(thread =0; thread < thread_count; thread++)
	{
		pthread_join(thread_handles[thread], NULL);
	}	
	free(thread_handles);
		
	printf("the result is %.6f\n",final_result);
	return 0;
}

// Determines how many of N each thread should do.
int divideWork(int thread_count)
{
	if(N % thread_count != 0)
	{
		printf("Thread count not divisible by number of tasks! aborting\n");
		exit(-1);
	}
	else
		return N / thread_count;
}

//Bakes in the relevant data for each thread.
void createArguments(struct arguments *args, double *final_res, double start, double end, int pieces)
{
	args->start = start;
	args->end = end; 
	args->final_res = final_res;
	args->num_pieces = pieces;
}

double f(double x) 
{
	return invsqrt2pi*exp(-0.5*x*x);
}

//return area for the given x-range 
double simpson(double x1,double x2,double x3) 
{
	return (x2-x1)*(f(x1)+4*f(x2)+f(x3))/3;
}


//calculate the integral using simpson's method.
//start,end: endpoints of interval to integrate over
//num_pieces: number of trapezoids the interval is divided into
void* integral(void *arg) 
{
	struct arguments* args = (struct arguments*)arg;
	double start = args -> start;
	double end = args -> end;
	int num_pieces = args -> num_pieces;
	double *final_result = args -> final_res;
	double res = 0.0;
	double h = (end-start)/num_pieces;
	
	for(int i=0;i<num_pieces;i+=2) 
	{
		res += simpson(start+i*h, start+(i+1)*h, start+(i+2)*h);
	}
	
	pthread_mutex_lock(&mutex1);
	*final_result += res;
	pthread_mutex_unlock(&mutex1);
	
	return NULL;
}
